import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.io.InputStream;
import java.nio.FloatBuffer;
import java.util.ArrayList;
import javax.media.opengl.GL;
import javax.media.opengl.GL2;
import javax.media.opengl.GL2ES1;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLCapabilities;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.GLProfile;
import javax.media.opengl.awt.GLCanvas;
import javax.media.opengl.glu.GLU;
import javax.swing.JFrame;
import com.jogamp.opengl.util.Animator;
import com.jogamp.opengl.util.texture.Texture;
import com.jogamp.opengl.util.texture.TextureData;
import com.jogamp.opengl.util.texture.TextureIO;



public class ParticleRenderer implements GLEventListener {


	private Texture texture;
	public final int MAX_PARTICLES = 5000;

	private ArrayList<Particle> particles = new ArrayList<Particle>();
	private Particle particle;

	private float[] ambientLight = {0.5f, 0.5f, 0.5f, 1.0f};
	private FloatBuffer lightAmbient = FloatBuffer.wrap(ambientLight);
	private float[] diffuseLight = {1.0f, 1.0f, 1.0f, 1.0f};
	private FloatBuffer lightDiffuse = FloatBuffer.wrap(diffuseLight);
	private float[] positionLight = {0.0f, 0.0f, 2.0f, 1.0f};
	private FloatBuffer lightPosition = FloatBuffer.wrap(positionLight);

	public static void main(String[]args){

		GLProfile glp = GLProfile.getDefault();
		GLCapabilities caps = new GLCapabilities(glp);
		GLCanvas canvas = new GLCanvas(caps);
		canvas.addGLEventListener(new ParticleRenderer());

		final Animator animator = new Animator(canvas);
		animator.add(canvas);
		animator.start();

		JFrame frame = new JFrame("Particles");
		frame.setSize(800, 800);
		frame.addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				new Thread(){
					public void run(){
						animator.stop();
						System.exit(0);
					}
				}.start();
			}
		});
		frame.add(canvas);
		frame.setVisible(true);
		frame.pack();

	}

	private void readTexture(GLAutoDrawable drawable){

		GL2 gl = drawable.getGL().getGL2();
		Texture texture = null;

		try {
			InputStream stream = getClass().getResourceAsStream("water.png");
			TextureData data = TextureIO.newTextureData(gl.getGLProfile(), stream, false, "jpg");
			texture = TextureIO.newTexture(data);
		} catch (IllegalArgumentException | IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		this.texture = texture;

	}

	private void createParticles(int count){
		count = Math.min(count, MAX_PARTICLES);
		for(int i=0; i<count;i++){
			particles.add(new Particle());
		}
	}


	private void update(){
		for(int i=0;i<particles.size();i++){		
			particle = particles.get(i);
			particle.evolve();
		}
	}

	private void render(GLAutoDrawable drawable){

		GL2 gl = drawable.getGL().getGL2();

		gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);

		gl.glBegin(GL2.GL_TRIANGLE_STRIP);
		gl.glNormal3d(0,1,0);
		gl.glVertex3d(-3, 0, 3);
		gl.glVertex3d(3, 0, 3);
		gl.glVertex3d(-3, 0, -3);
		gl.glVertex3d(3, 0, -3);
		gl.glEnd();




		for(int i=0;i<Math.min(particles.size(),MAX_PARTICLES);i++){

			particle = particles.get(i);

			texture.enable(gl);
			texture.bind(gl);

			gl.glBegin(GL2.GL_TRIANGLE_STRIP);

			gl.glNormal3d(0,0,1);   
			gl.glTexCoord2f(0.0f, 0.0f);
			gl.glVertex3f(particle.xpos+particle.PARTICLE_SIZE, particle.ypos+particle.PARTICLE_SIZE, particle.zpos+0.0f);

			gl.glTexCoord2f(1.0f, 0.0f);
			gl.glVertex3f(particle.xpos-particle.PARTICLE_SIZE, particle.ypos+particle.PARTICLE_SIZE, particle.zpos+0.0f);

			gl.glTexCoord2f(0.0f, 1.0f);
			gl.glVertex3f(particle.xpos+particle.PARTICLE_SIZE, particle.ypos-particle.PARTICLE_SIZE, particle.zpos+0.0f);

			gl.glTexCoord2f(1.0f, 1.0f);
			gl.glVertex3f(particle.xpos-particle.PARTICLE_SIZE, particle.ypos-particle.PARTICLE_SIZE, particle.zpos+0.0f);

			gl.glEnd();


			texture.disable(gl);

		}


	}

	@Override
	public void display(GLAutoDrawable drawable) {

		update();
		render(drawable);

	}

	@Override
	public void dispose(GLAutoDrawable arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public void init(GLAutoDrawable drawable) {

		readTexture(drawable);
		createParticles(10000);

		GL2 gl = drawable.getGL().getGL2();
		gl.glEnable(GL.GL_TEXTURE_2D);
		gl.glShadeModel(GL2.GL_SMOOTH);//Smooth Shading
		gl.glClearColor(1.0f, 1.0f, 1.0f, 0.0f);//Black background
		gl.glClearDepth(1.0);//Depth Buffer Setup
		gl.glEnable(GL.GL_DEPTH_TEST);//Enables Depth Testing
		gl.glDepthFunc(GL.GL_LEQUAL);//The Type of Depth Test 
		gl.glHint(GL2ES1.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);//Perspective calculations

		gl.glLightfv(GL2.GL_LIGHT1, GL2.GL_AMBIENT, lightAmbient);
		gl.glLightfv(GL2.GL_LIGHT1, GL2.GL_DIFFUSE, lightDiffuse);
		gl.glLightfv(GL2.GL_LIGHT1, GL2.GL_POSITION, lightPosition);

		gl.glEnable(GL2.GL_LIGHTING);
		gl.glEnable(GL2.GL_LIGHT1);

		GLU glu = new GLU();
		gl.glMatrixMode(GL2.GL_MODELVIEW);
		gl.glLoadIdentity();

		glu.gluLookAt(1,1, 2.5, -1.5, 0, -5, 0, 1, 0);

	}

	@Override
	public void reshape(GLAutoDrawable drawable, int x, int y, int w,
			int h) {

		GL2 gl = drawable.getGL().getGL2();
		GLU glu = GLU.createGLU();

		gl.glMatrixMode(GL2.GL_PROJECTION); //select the projection matrix
		gl.glLoadIdentity(); //reset the projection matrix
		glu.gluPerspective(45.0f, (float)w/h, 0.1f, 100.0f);//calculate aspect ration of the window


	}

}


