import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Point2D;
import java.nio.FloatBuffer;
import java.util.ArrayList;

import javax.swing.*;
import javax.media.opengl.*;
import javax.media.opengl.awt.GLCanvas;
import javax.media.opengl.glu.GLU;
import javax.media.opengl.glu.GLUquadric;

import com.jogamp.opengl.util.Animator;



public class PathAnimation extends JFrame{

	GLCanvas canvas;
	SceneView sv;

	float posx,posy,posz;
	int frame = 0;


	public PathAnimation() {

		sv = new SceneView(); //GLEventListener-Objekt

		GLProfile glp = GLProfile.getDefault();
		GLCapabilities caps = new GLCapabilities(glp);
		canvas = new GLCanvas(caps);

		canvas.addGLEventListener(sv);
		final Animator animator = new Animator(canvas);
		animator.add(canvas);
		animator.start();
		add(canvas);
		addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				new Thread(){
					public void run(){
						animator.stop();
						System.exit(0);
					}
				}.start();
			}
		});

		setTitle("Casteljau");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setSize(500,500);
		setVisible(true);  

	}



	class SceneView implements GLEventListener {

		Casteljau casteljau = new Casteljau();
		ArrayList<Point2D.Double> ctrlPoints = casteljau.getCtrlPoints();
		ArrayList<Point2D.Double> curvePoints = casteljau.getCurvePoints();

		float no_mat[] = { 0.0f, 0f, 0f, 1.0f };
		float mat_diffuse[] = { 0.1f, 0.5f, 0.8f, 1.0f };
		float mat_specular[] = { 0.6f, 0.2f, 0.6f, 1.0f };
		float low_shininess[] = { 10.0f };


		public void init(GLAutoDrawable drawable) {
			GL2 gl = drawable.getGL().getGL2(); 
			gl.glClearColor(1, 1, 1, 0f); // grey background	
			//light setup
			float light_ambient[] = { 0.2f, 0.2f, 0.2f, 1.0f };
			float light_diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
			float light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
			float light_position[] = { -10f, -6f, 10.0f, 0.0f };
			float light_direction[] = { 10f, 5f, -10.0f, 0.0f };


			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_AMBIENT, FloatBuffer.wrap(light_ambient));
			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_DIFFUSE, FloatBuffer.wrap(light_diffuse));
			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_SPECULAR, FloatBuffer.wrap(light_specular));
			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_POSITION, FloatBuffer.wrap(light_position));
			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_SPOT_DIRECTION, FloatBuffer.wrap(light_direction));

			gl.glEnable(GL2.GL_LIGHTING);
			gl.glEnable(GL2.GL_LIGHT0);

			gl.glEnable(GL.GL_DEPTH_TEST);
			gl.glClearDepth(1.0f);                      
			gl.glDepthFunc(GL.GL_LEQUAL);
			gl.glHint(GL2.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);
			gl.glShadeModel(GL2.GL_SMOOTH); //Gouraud-Shading

		}

		public void display(GLAutoDrawable drawable) {

			update();



			GL2 gl = drawable.getGL().getGL2();         
			gl.glClear(GL2.GL_COLOR_BUFFER_BIT|GL2.GL_DEPTH_BUFFER_BIT); // clear background
			GLU glu = new GLU();

			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_AMBIENT, FloatBuffer.wrap(no_mat));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_DIFFUSE, FloatBuffer.wrap(mat_diffuse));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_SPECULAR, FloatBuffer.wrap(mat_specular));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_SHININESS, FloatBuffer.wrap(low_shininess));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_EMISSION, FloatBuffer.wrap(no_mat));


			//quadric with normals and texture coordinates
			GLUquadric q = glu.gluNewQuadric();
			glu.gluQuadricDrawStyle( q, GLU.GLU_FILL );
			glu.gluQuadricNormals( q, GLU.GLU_SMOOTH );
			glu.gluQuadricTexture(q, true);

			//gl.glColor3d(0, 0, 1);	//blue
			gl.glBegin(GL2.GL_LINE_STRIP); 
			for(int i = 0; i<curvePoints.size();i++){	
				gl.glVertex3d(curvePoints.get(i).x, curvePoints.get(i).y, 0);
			}
			gl.glEnd();

			//gl.glColor3d(1, 0, 0);	//red
			gl.glPointSize(5.0f);
			gl.glBegin(GL2.GL_POINTS); 
			for(int i = 0; i<ctrlPoints.size();i++){
				gl.glVertex3d(ctrlPoints.get(i).x, ctrlPoints.get(i).y, 0);
			}
			gl.glEnd();

			gl.glBegin(GL2.GL_LINE_STRIP); 
			for(int i = 0; i<ctrlPoints.size();i++){
				gl.glVertex3d(ctrlPoints.get(i).x, ctrlPoints.get(i).y, 0);
			}
			gl.glEnd();

			gl.glPushMatrix();
			gl.glTranslatef(posx, posy, posz);
			glu.gluSphere(q, 0.5f, 200, 200);
			gl.glPopMatrix();
		}



		public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h) {
			GL2 gl = drawable.getGL().getGL2();
			gl.glViewport(Math.max(0,(w-h)/2), Math.max(0,(h-w)/2), Math.min(w,h), Math.min(w,h)); //reset the current viewport
			gl.glMatrixMode(GL2.GL_PROJECTION);
			gl.glLoadIdentity();
			gl.glOrtho(-1,15,-8,8,-1,100);
		}

		public void dispose(GLAutoDrawable drawable) {
		}

		private void update(){
			posx = (float)curvePoints.get(frame%curvePoints.size()).x;
			posy = (float)curvePoints.get(frame%curvePoints.size()).y;

			frame ++;
		}


	}

	public static void main(String args[]) {
		new PathAnimation();   
	}




}


